/*
 * Copyright (c) 2013 F. Mannhardt (f.mannhardt@tue.nl)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
package org.processmining.xeslite.external;

import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeMap;

/**
 * @author F. Mannhardt
 * 
 */
class XEventCachingExternalImpl extends XEventBareExternalImpl implements AttributesCacheable {

	private String name = null;
	private String transition = null;
	private long time = -1;

	XEventCachingExternalImpl(ExternalStore store) {
		this(null, store);
	}

	XEventCachingExternalImpl(XAttributeMap attributeMap, ExternalStore store) {
		super(null, store);
		// Need to initialize our attributes ourselves as otherwise the cached fields get overwritten by our initializer  
		initAttributes(attributeMap);
	}

	XEventCachingExternalImpl(long id, XAttributeMap attributeMap, ExternalStore store) {
		super(id, null, store);
		initAttributes(attributeMap);
	}

	private final void initAttributes(XAttributeMap attributeMap) {
		if (attributeMap != null) {
			XAttributeMap newAttributeMap = getAttributes();
			for (XAttribute a: attributeMap.values()) {
				newAttributeMap.put(a.getKey(), a);
			}
		}
	}

	@Override
	public boolean hasAttributes() {
		// Use the cached attribute as indicator, if available 
		if (time != -1 || name != null || transition != null) {
			return true;
		}
		return super.hasAttributes();
	}

	@Override
	public Object clone() {
		return new XEventCachingExternalImpl(getAttributes(),  getStore());
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getName() {
		return name;
	}

	public void setTransition(String transition) {
		this.transition = transition;
	}

	public String getTransition() {
		return transition;
	}

	public void setTime(long value) {
		this.time = value;
	}

	public long getTime() {
		return time;
	}

}