package org.processmining.xeslite.query;

import java.util.Date;

import org.deckfour.xes.model.XAttributable;
import org.deckfour.xes.model.XAttributeBoolean;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.processmining.xeslite.query.syntax.ParseException;

import com.googlecode.cqengine.attribute.Attribute;
import com.googlecode.cqengine.index.Index;
import com.googlecode.cqengine.query.Query;
import com.googlecode.cqengine.resultset.ResultSet;

public interface XIndexedAttributable<T extends XAttributable> {

	/**
	 * @param index
	 */
	public abstract void addIndex(Index<T> index);

	/**
	 * @param query
	 * @return
	 */
	public abstract ResultSet<T> retrieve(Query<T> query);

	/**
	 * @param query
	 * @return
	 * @throws ParseException
	 */
	Query<T> parseQuery(String query) throws ParseException;

	/**
	 * @param query
	 * @param attributeTypeResolver
	 * @return
	 * @throws ParseException
	 */
	Query<T> parseQuery(String query, AttributeTypeResolver attributeTypeResolver) throws ParseException;
	
	/**
	 * @param query
	 * @return
	 * @throws ParseException
	 */
	public abstract ResultSet<T> retrieve(String query) throws ParseException;

	/**
	 * @param element
	 * @return
	 */
	public abstract boolean remove(T element);

	/**
	 * Add an {@link XAttributable}
	 *
	 * @param element
	 */
	public abstract void add(T element);


	/**
	 * String attribute for {@link XAttributeLiteral} and {@link XAttributeBoolean}
	 *
	 * @param attributeName
	 * @return
	 */
	public Attribute<T, String> stringAttribute(String attributeName);

	/**
	 * Double attribute for {@link XAttributeContinuous} and {@link XAttributeDiscrete}
	 *
	 * @param attributeName
	 * @return
	 */
	public Attribute<T, Double> doubleAttribute(String attributeName);

	/**
	 * Date attribute for {@link XAttributeTimestamp}
	 *
	 * @param attributeName
	 * @return
	 */
	public Attribute<T, Date> dateAttribute(String attributeName);

}